@@ -13,7 +13,7 @@ import_packaged_environments = boolean()
 status_performance_warn_threshold = float(default=0.2)
 show_traceback = boolean()
 eligible_jobs_max_lines = int(default=10)
-status_parallelization = string(default='thread')
+status_parallelization = string(default='none')
 """
 
 
